<?php

/**
 * Expose menu items as context reactions.
 */
class context_reaction_alter_form extends context_reaction {
  /**
   * Provide a form elements allowing adding fields and values.
   */
  function options_form($context) {
    // Initalize.
    $form = array();
    $delta = 0;
    $notes = t('<h3>Alter Forms</h3>
      Parent heirarchy should be listed including the target element (comma separated), eg: "group_THIS, field_THIS". <br>
      Attributes should use Drupal form API norms, eg: #default_value, #title, #maxlength. See !api_link. <br><br>',
      array('!api_link' => l(
        t('Form API Reference'),
        'http://api.drupal.org/api/drupal/developer%21topics%21forms_api_reference.html/7'
      ))
    );
    $form['elements'] = array(
      '#theme' => 'context_form_alteration_settings_table',
      '#prefix' => '<div id="context-form-alteration-elements-table">' . $notes,
      '#suffix' => '</div>',
    );

    // Build existing entries.
    $values = $this->fetch_from_context($context);
    foreach ($values['elements'] as $val) {
      $form['elements'][$delta] = _context_form_alteration_build_row($delta, $val['form_id'], $val['parents'], $val['attribute'], $val['value']);
      $delta++;
    }

    // Add placeholder rows
    $number_of_empty_rows = 3;
    for ($i = 0; $i < $number_of_empty_rows; $i++) {
      $form['elements'][$delta] = _context_form_alteration_build_row($delta);
      $delta++;
    }

    // Add space for a new addition.
    // @todo Allow adding more. BEWARE: There is no $form_state
    // http://drupal.stackexchange.com/questions/32152/add-another-item-for-custom-forms
    // http://drupalcode.org/project/flush_page_cache.git/blob/refs/heads/7.x-1.x:/flush_page_cache.admin.inc

/*
    $form['reaction_alteration'] = array(
      '#type' => 'button',
      '#value' => t('Add Element'),
      '#ajax' => array(
        'callback' => '_context_form_alteration_callback',
        'wrapper' => 'context-form-alteration-settings-table',
        'method' => 'replace',
        'effect' => 'fade',
      ),
    );
*/
    return $form;
  }

  /**
   * Override of options_form_submit().
   */
  function options_form_submit($values) {    
    // Clean up empty values for storage in neat, tidy array.
    foreach ($values['elements'] as $key => $option)  {
      if($option['form_id'] == '' || $option['parents'] == '' || $option['attribute'] == '') {
        unset($values['elements'][$key]);
      }
    }
    
    return $values;
  }

  function execute(&$form) {
    $contexts = $this->get_contexts();
    foreach ($contexts as $context) {
      if (!empty($context->reactions[$this->plugin])) {
        foreach ($context->reactions[$this->plugin] as $option_id => $option) {

          // Whether we should bother.
          foreach ($option as $opt) {            
            if ($opt['form_id'] === $form['#form_id']) {
              // Alter the form value.
              // @todo Add option for forcing.
              $parents = array();
              $parents = explode(',', $opt['parents']);
              $parents[] = $opt['attribute'];
              // Trim up the values, allowing messier/easier user input.
              array_walk($parents, create_function('&$val', '$val = trim($val);'));
              drupal_array_set_nested_value($form, $parents, $opt['value']);
            }
          }

        }
      }
    }
  }

}
